/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* resolver.c - resolver hooks for libproxynet
 * vix 27apr94 [original]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: resolver.c,v 1.4 1996/05/16 04:48:39 vixie Exp $";
#endif

#include "vixutil.h"

#include <arpa/nameser.h>
#include <resolv.h>

#include "defs.h"

static struct {
	void			*pcb;
	int			save_nscount;
	struct sockaddr_in	save_nsaddr_list[MAXNS];
} State;

/*
 * Query hook.
 */
static res_sendhookact
qhook(struct sockaddr_in * const *ns,
      const char **query,
      int *querylen,
      char *ans,
      int anssiz,
      int *resplen)
{
	HEADER *hp = (HEADER *)*query;
	const char *cp = *query + sizeof(HEADER);
	char name[MAXDNAME];
	void *pcb;
	int n;

	if (hp->opcode != QUERY || ntohs(hp->qdcount) != 1)
		return (res_goahead);

	n = dn_expand((u_char*)*query, (u_char*)cp+MAXCDNAME, (u_char*)cp,
		      (u_char*)name, sizeof(name));
	if (n < 0)
		return (res_goahead);
	cp += n;

	pcb = proxyThisName(name);
	if (pcb == State.pcb) {
		/*
		 * No change, same proxy (maybe none) as before.
		 */
		return (res_goahead);
	}
	if (!pcb) {
		/*
		 * We were proxying, now we're not.
		 */
		proxresReset();
		return (res_goahead);
	}
	if (!State.pcb) {
		/*
		 * We weren't proxying, now we are.
		 */
		memcpy(State.save_nsaddr_list, _res.nsaddr_list,
		       sizeof State.save_nsaddr_list);
		State.save_nscount = _res.nscount;
	}
	/*
	 * Install new proxy's name servers.
	 */
	proxyOverrideNS(&_res, pcb);
	State.pcb = pcb;
	return (res_goahead);
}


/*
 * Response hook.
 */
static res_sendhookact
rhook(const struct sockaddr_in *ns,
      const char *query,
      int querylen,
      char *ans,
      int anssiz,
      int *resplen)
{
	return (res_goahead);
}


void
proxresInit() {
	static int initFlag = 0;

	if (!initFlag) {
		initFlag++;
		res_send_setqhook(qhook);
		res_send_setrhook(rhook);
	}
}


void
proxresReset() {
	if (State.pcb) {
		memcpy(_res.nsaddr_list, State.save_nsaddr_list,
		       sizeof State.save_nsaddr_list);
		_res.nscount = State.save_nscount;
		State.pcb = NULL;
	}
}
