/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sys_accept.c - "accept" syscall replacement for libproxynet
 * vix 14may94 [cloned from sys_getpeername.c (21mar94)]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: sys_accept.c,v 1.7 1996/05/16 04:48:39 vixie Exp $";
#endif

#include "vixutil.h"

#include <sys/syscall.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "defs.h"
#include "messages.h"


int
acceptSys(int s, struct sockaddr *name, int *namelen) {
	int ret = syscall(SYS_accept, s, name, namelen);

	if (name && name->sa_family == AF_INET)
		proxyTr(PROXTRC_SYSCALL, "acceptSys(%d, %s/%d, %d)\n",
			s, inet_ntoa(((struct sockaddr_in*)name)->sin_addr),
			ntohs(((struct sockaddr_in*)name)->sin_port),
			namelen ? *namelen : -1);
	return (ret);
}


int
accept(int s, struct sockaddr *name, int *namelen) {
	struct sock peer;
	int ret, save;
	void *pcb;

#ifdef NOT_YET
	if (proxfdGetT(s) == fdTlisten) {
		dwim(now we need to build an fd_set with bits set in the
		     read mask for "s" and all session descriptors of PCBs
		     which are proxying for this socket.)
		dwim(then we need to select on this set.  no timeout is needed,
		     since a proxy session socket is never readable while in
		     command input phase, unless it has died and is EOFing.)
		dwim(any bit set other than "s" should be considered a dead
		     proxynet server, and we should downcall into common code
		     that closes and reopens all connections and listeners on
		     any such server.)
		dwim(if select() returns 1 and only "s" bit it set, fall
		     through to the acceptSys().)
		dwim(otherwise loop here)
	}
#endif /*NOT_YET*/
	ret = acceptSys(s, name, namelen);
	if (ret != -1) {
		proxfdSet(s, ret);
		proxfdSetT(ret, fdTaccept);
	}
	if (!name || !namelen)
		return (ret);
	save = errno;

	peer.addr = *name;
	if (ret != -1 &&
	    (peer.type = sock_fdtype(s)) != -1 &&
	    (pcb = proxyAddrIsProxy(&peer))) {
		int ret2 = getpeernameProxy(s, &peer, pcb);

		if (ret2 != -1) {
			/* XXX - name->sa_family may not make sense for s */
			*name = peer.addr;
			/* XXX - maybe need to be type-specific here? */
			*namelen = sock_len(&peer);
			proxfdSetP(ret, pcb);
		} else {
			proxyLog("accept: getpeernameProxy: %m");
			errno = save;
		}
	}
	return (ret);
}
