/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sys_bind.c - "bind" system call replacement for libproxynet
 * vix 30jun94 [cloned from sys_close.c [27apr94]]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: sys_bind.c,v 1.3 1996/05/16 04:48:39 vixie Exp $";
#endif

#include "vixutil.h"

#include <sys/syscall.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "defs.h"


/*
 * The goal here is to catch wildcard service bindings so that we can
 * proxy that semantic if need be.
 */


int
bindSys(int s, const struct sockaddr *name, int len) {
	if (name->sa_family == AF_INET)
		proxyTr(PROXTRC_SYSCALL, "bindSys(%d, %s/%d, %d)\n",
			s, inet_ntoa(((struct sockaddr_in*)name)->sin_addr),
			ntohs(((struct sockaddr_in*)name)->sin_port), len);
	return (syscall(SYS_bind, s, name, len));
}


int
bind(int s, const struct sockaddr *name, int len) {
	int ret;

	(void) proxyInit();
	ret = bindSys(s, name, len);
	if (ret != -1) {
		if (name->sa_family == AF_INET &&
		    ntohs(((struct sockaddr_in*)name)->sin_port) == 0)
			proxfdSetW(s, TRUE);
		else
			proxfdSetW(s, FALSE);
	}
	return (ret);
}
