/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sys_close.c - "close" system call replacement for libproxynet
 * vix 27apr94 [cloned from sys_listen [30mar94]]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: sys_close.c,v 1.4 1996/05/16 04:48:39 vixie Exp $";
#endif

#include "vixutil.h"

#include <sys/syscall.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "defs.h"


/*
 * This is done partly to cause the initialization as early as possible;
 * we have no other way to insure that the resolver hooks are installed
 * before they are used.  We happen to know that res_init() calls close(),
 * which makes the resolver somewhat reentrant since we call the hook
 * installers, but we happen to know that they are light weight and won't
 * mind being called from res_init()'s stack.
 *
 * We also take this opportunity to clear out this fd's hints, if any.
 */


int
closeSys(int fd) {
	if (sock_fdtype(fd) != -1)
		proxyTr(PROXTRC_SYSCALL, "closeSys(%d)\n", fd);
	return (syscall(SYS_close, fd));
}


int
close(int fd) {
	(void) proxyInit();
	proxfdClr(fd);
	return (closeSys(fd));
}
