/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sys_connect.c - "connect" system call replacement for libproxynet package
 * vix 02feb94 [original]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: sys_connect.c,v 1.8 1996/05/16 04:48:39 vixie Exp $";
#endif

#include "vixutil.h"

#include <sys/syscall.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "defs.h"
#include "messages.h"


int
connectSys(int s, const struct sockaddr *name, int namelen) {
	if (name->sa_family == AF_INET)
		proxyTr(PROXTRC_SYSCALL, "connectSys(%d, %s/%d, %d)\n",
			s, inet_ntoa(((struct sockaddr_in*)name)->sin_addr),
			ntohs(((struct sockaddr_in*)name)->sin_port), namelen);
	return (syscall(SYS_connect, s, name, namelen));
}


static int
connectProxy(int s, struct sock *dest, void *pcb) {
	static const char funcName[] = "connectProxy";
	char buf[ARPA_MAXSTR], tmp[MEDIUM];
	FILE *resp;
	int code, ret;

#if notyet	/* XXX */
	/*
	 * Remake socket in a type which can be used to reach the proxy
	 * server, if it is not already of that type.  Dangerous since
	 * the app may do a getsockname() and not understand how the type
	 * could have changed from the time of its own socket() call.
	 */
	if (dest->type != proxySessType(pcb)) {
		int s1;

		dest->type = proxySessType(pcb);
		if ((s1 = socket(dest->type, SOCK_STREAM, PF_UNSPEC)) == -1)
			return (-1);
		if (dup2(s1, s) == -1)
			return (-1);
		close(s1);
	}
#endif

	sprintf(buf, "conn %s", sock_ntoa(dest));
	if (!(resp = proxyTransaction(pcb, buf))) {
		syslog(LOG_INFO, proxmsg_CantConn,
		       funcName, sock_ntoa(dest));
		errno = ENETUNREACH;
		return (-1);
	}
	while (!proxsrvResp(resp, buf, sizeof buf, &code))
		NULL;
	if ((code & 0700) != 0200) {	/* XXX - need to check entire code? */
		syslog(LOG_INFO, proxmsg_RespNotOK,
		       funcName, sock_ntoa(dest), buf);
		errno = ENETUNREACH;
		return (-1);
	}
	if (arpa_parsefield(buf, tmp, sizeof tmp) <= 0) {
		syslog(LOG_WARNING, proxmsg_RespBadFmt,
		       funcName, sock_ntoa(dest), buf);
		errno = ECONNABORTED;
		return (-1);
	}
	if (!sock_aton(tmp, dest)) {
		syslog(LOG_WARNING, proxmsg_RespBadAddr,
		       funcName, sock_ntoa(dest), tmp);
		errno = ECONNABORTED;
		return (-1);
	}
	ret = connectSys(s, &dest->addr, sizeof dest->addr);
	if (ret != -1) {
		proxfdSetT(s, fdTconnect);
		proxfdSetP(s, pcb);
	}
	return (ret);
}


int
connect(int s, const struct sockaddr *name, int namelen) {
	struct sock dest;
	void *pcb;
	int ret;

	dest.addr = *name;
	if ((dest.type = sock_fdtype(s)) != -1 &&
	    (pcb = proxyThisAddr(&dest)) &&
	    proxyProxsrvs_p(pcb))
		return (connectProxy(s, &dest, pcb));
	ret = connectSys(s, name, namelen);
	return (ret);
}
