/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sys_getpeername.c - "getpeername" syscall replacement for libproxynet
 * vix 21mar94 [cloned from connect.c vix 02feb94 [original]]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: sys_getpeername.c,v 1.4 1996/05/16 04:48:39 vixie Exp $";
#endif

#include "vixutil.h"

#include <sys/syscall.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "defs.h"
#include "messages.h"


static const char getpeernameProxy__Name[] = "getpeernameProxy";


int
getpeernameSys(int s, struct sockaddr *name, int *namelen) {
	int ret = syscall(SYS_getpeername, s, name, namelen);

	if (name->sa_family == AF_INET)
		proxyTr(PROXTRC_SYSCALL, "getpeernameSys(%d, %s/%d, %d)\n",
			s, inet_ntoa(((struct sockaddr_in*)name)->sin_addr),
			ntohs(((struct sockaddr_in*)name)->sin_port),
			*namelen);
	return (ret);
}


int
getpeernameProxy(int s, struct sock *peer, void *pcb) {
	char buf[ARPA_MAXSTR], tmp[MEDIUM];
	FILE *resp;
	int code;

	/* XXX:	there's an awful lot of common code here */

	/* XXX:	need to guess client/server side based on previously
	 *	interrupted syscalls (connect vs accept); for now we
	 *	just assume server side which is very simple-minded.
	 * XXX!	What the hell was I talking about here?  (vix, June94)
	 */

	sprintf(buf, "find cpa %s sra", sock_ntoa(peer));
	if (!(resp = proxyTransaction(pcb, buf))) {
		syslog(LOG_INFO, proxmsg_CantConn,
		       getpeernameProxy__Name, sock_ntoa(peer));
		errno = ENETUNREACH;
		return (-1);
	}
	while (!proxsrvResp(resp, buf, sizeof buf, &code))
		NULL;
	if ((code & 0700) != 0200) {	/* XXX - need to check entire code? */
		syslog(LOG_INFO, proxmsg_RespNotOK,
		       getpeernameProxy__Name, sock_ntoa(peer), buf);
		errno = ENETUNREACH;
		return (-1);
	}
	if (arpa_parsefield(buf, tmp, sizeof tmp) <= 0) {
		syslog(LOG_WARNING, proxmsg_RespBadFmt,
		       getpeernameProxy__Name, sock_ntoa(peer), buf);
		errno = ECONNABORTED;
		return (-1);
	}
	if (!sock_aton(tmp, peer)) {
		syslog(LOG_WARNING, proxmsg_RespBadAddr,
		       getpeernameProxy__Name, sock_ntoa(peer), tmp);
		errno = ECONNABORTED;
		return (-1);
	}
	return (0);
}


int
getpeername(int s, struct sockaddr *name, int *namelen) {
	int ret, save;
	struct sock peer;
	void *pcb;

	memset(name, 0, sizeof *name);
	ret = getpeernameSys(s, name, namelen);
	save = errno;

	peer = sock_sock(sock_fdtype(s), name);

	if (ret != -1 &&
	    (pcb = proxyAddrIsProxy(&peer))) {
		int ret2 = getpeernameProxy(s, &peer, pcb);

		if (ret2 != -1) {
			/* XXX - name->sa_family may not make sense for s */
			*name = peer.addr;
			/* XXX - maybe need to be type-specific here? */
			*namelen = sock_len(&peer);
			return (ret2);
		}
		proxyLog("%s: %m", getpeernameProxy__Name);
		errno = save;
	}
	return (ret);
}
