/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sys_getproxname.c - "getproxname" utility function, similar to getsockname
 * vix 29jun94 [cloned from getpeername.c vix 21mar94 [cloned]]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: sys_getproxname.c,v 1.4 1996/05/16 04:48:39 vixie Exp $";
#endif

#include "vixutil.h"

#include <sys/syscall.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "defs.h"
#include "messages.h"


static const char getsocknameProxy__Name[] = "getsocknameProxy";


int
getsocknameSys(int s, struct sockaddr *name, int *namelen) {
	int ret = syscall(SYS_getsockname, s, name, namelen);

	if (name->sa_family == AF_INET)
		proxyTr(PROXTRC_SYSCALL, "getsocknameSys(%d, %s/%d, %d)\n",
			s, inet_ntoa(((struct sockaddr_in*)name)->sin_addr),
			ntohs(((struct sockaddr_in*)name)->sin_port),
			*namelen);
	return (ret);
}


int
getsocknameProxy(int s, struct sock *sock, void *pcb) {
	char buf[ARPA_MAXSTR], tmp[MEDIUM];
	FILE *resp;
	int code;

	sprintf(buf, "find cla %s spa", sock_ntoa(sock));
	if (!(resp = proxyTransaction(pcb, buf))) {
		syslog(LOG_INFO, proxmsg_CantConn,
		       getsocknameProxy__Name, sock_ntoa(sock));
		errno = ENETUNREACH;
		return (-1);
	}
	while (!proxsrvResp(resp, buf, sizeof buf, &code))
		NULL;
	if ((code & 0700) != 0200) {	/* XXX - need to check entire code? */
		syslog(LOG_INFO, proxmsg_RespNotOK,
		       getsocknameProxy__Name, sock_ntoa(sock), buf);
		errno = ENETUNREACH;
		return (-1);
	}
	if (arpa_parsefield(buf, tmp, sizeof tmp) <= 0) {
		syslog(LOG_WARNING, proxmsg_RespBadFmt,
		       getsocknameProxy__Name, sock_ntoa(sock), buf);
		errno = ECONNABORTED;
		return (-1);
	}
	if (!sock_aton(tmp, sock)) {
		syslog(LOG_WARNING, proxmsg_RespBadAddr,
		       getsocknameProxy__Name, sock_ntoa(sock), tmp);
		errno = ECONNABORTED;
		return (-1);
	}
	return (0);
}


static int
eachPcb(void *pcb, int s, void *uap) {
	struct sock *sock = (struct sock *)uap;

	if (getsocknameProxy(s, sock, pcb) == -1)
		return (-1);
	return (1);	/* "stop here".  ambiguous as hell.  XXX. */
}


/* XXX:	This implementation is dangerously ambiguous since it assumes that
 *	any local socket which is the target of a proxy listener is a proxy
 *	socket -- getproxname() will return an address on the proxy host.
 *	This won't hurt ftp since it's fairly short lived.  It will kill
 *	bigger things like INN or NameD which usefully accept connections
 *	in both the real and proxied universes.  Fixing this will require
 *	an interface change but we don't know yet (June94) what that will
 *	be.  (vix)
 */


int
getproxname(int s, struct sockaddr *name, int *namelen) {
	int save;
	struct sock sock;
	void *pcb;

	memset(name, 0, sizeof *name);
	if (getsocknameSys(s, name, namelen) == -1)
		return (-1);
	save = errno;
	sock = sock_sock(sock_fdtype(s), name);

	if (proxfdEachP(s, eachPcb, &sock) > 0) {
		/* XXX - name->sa_family may not make sense for s */
		*name = sock.addr;
		/* XXX - maybe need to be type-specific here? */
		*namelen = sock_len(&sock);
		return (0);
	}
	errno = save;
	return (0);
}
