/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* addrhint.c - code for the address hint package
 * vix 22sep93 [original]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: addrhint.c,v 1.4 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include <stdio.h>
#include <stdlib.h>

struct hint {
	ListLink(	struct hint)
	struct sock	addr;
	time_t		timeout;
	void		*userdata;
	int		flags;
};

static ListHeadV(Hints, struct hint);

static void
free_userdata(struct hint *hint) {
	if (hint->userdata && (hint->flags & ADDRHINT_DYN))
		free(hint->userdata);
}

static void
release(struct hint *hint) {
	free_userdata(hint);
	free(hint);
}

int
addrhint_set(struct sock addr, time_t timeout, void *userdata, int flags) {
	struct hint	*hint;
	int		ret;

	ret = FALSE;
	ListEach(&Hints, hint) {
		if (sock_same(&addr, &hint->addr)) {
			ret = TRUE;
			break;
		}
	}
	if (hint) {
		if (hint->userdata != userdata)
			free_userdata(hint);
	} else {
		hint = ZALLOC(struct hint, ZAL_ZERO|ZAL_ASSERT);
		ListAdd(&Hints, hint);
	}
	hint->addr = addr;
	hint->timeout = timeout;
	hint->userdata = userdata;
	hint->flags = flags;
	return (ret);
}

int
addrhint_clr(struct sock addr) {
	struct hint	*hint, *next, *prev;
	int		ret;

	ret = FALSE;
	prev = NULL;
	ListEach2(&Hints, hint, next) {
		if (sock_same(&addr, &hint->addr)) {
			ListUnlink(&Hints, hint, prev);
			release(hint);
			ret = TRUE;
			continue;
		}
		prev = hint;
	}
	return (ret);
}

int
addrhint_qry(struct sock addr, time_t *timeout, void **userdata, int *flags) {
	time_t		now = time(NULL);
	struct hint	*hint, *prev, *next;
	int		ret;

	ret = FALSE;
	prev = NULL;
	ListEach2(&Hints, hint, next) {
		if (hint->timeout <= now) {
			ListUnlink(&Hints, hint, prev);
			release(hint);
			continue;
		}
		if (sock_same(&addr, &hint->addr)) {
			if (timeout)
				*timeout = hint->timeout;
			if (userdata)
				*userdata = hint->userdata;
			if (flags)
				*flags = hint->flags;
			ret = TRUE;
		}
		prev = hint;
	}
	return (ret);
}
