/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* arpa_parsefield.c - parser for arpa-style <field>'s within strings
 * vixie 08may94 [original]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: arpa_parsefield.c,v 1.5 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include <string.h>
#include <errno.h>

/*
 * Parse a string which might contain a "<"field">", and get that field.
 *
 * Arguments:
 *	src	where to look
 *	dst	where to store null-terminated result
 *	size	size of dst
 *
 * Return values:
 *	-1 = error; look at errno
 *		(ENAMETOOLONG on dst overflow)
 *		(ENOENT if the '<' or '>' can't be found)
 *	0  = no field was found
 *	>0 = found a field, skipped this many characters to get past the '>'
 */

int
arpa_parsefield(const char *src, char *dst, int size) {
	char *l, *r;
	int len;

	if (!(l = strchr(src, '<')) ||
	    !(r = strchr(l+1, '>'))) {
		errno = ENOENT;
		return (-1);
	}
	len = r - l - 1;
	if (len > size-1) {
		errno = ENAMETOOLONG;
		return (-1);
	}
	(void) memcpy(dst, l+1, len);
	dst[len] = '\0';
	return (r - src + 1);
}
