/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* copy1way.c - copy unidirectionally on a pair of sockets
 * vixie 16may94 [cloned from copy2way.c (27apr94)]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: copy1way.c,v 1.4 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sysexits.h>

static const int on = 1;
static const struct linger ling = {1, 15};	/* arbitrary */

int
copy1way(int in, int out, char *buf, int bufsiz) {
	fd_set fds, rfds, efds;
	int x;

	if (setsockopt(out, SOL_SOCKET, SO_LINGER, &ling, sizeof ling) == -1)
		return (-1);
	if (setsockopt(in, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof on) == -1)
		return (-1);
	if (!buf) {
		errno = EINVAL;
		return (-1);
	}

#define UNINTR(s)  while ((s) == -1 && errno == EINTR)  NULL

	FD_ZERO(&fds);
	FD_SET(in, &fds);
	for (;;) {
		efds = fds, rfds = fds;
		UNINTR(x = select(in+1, &rfds, NULL, &efds, NULL));
		if (x <= 0)
			break;
		if (FD_ISSET(in, &efds)) {
			UNINTR(x = recv(in, buf, bufsiz, MSG_OOB));
			if (x <= 0)
				break;
			if ((x = send(out, buf, x, MSG_OOB)) <= 0)
				break;
		}
		if (FD_ISSET(in, &rfds)) {
			UNINTR(x = read(in, buf, bufsiz));
			if (x <= 0)
				break;
			if ((x = write(out, buf, x)) <= 0)
				break;
		}
	}
	if (x < 0)
		return (-1);
	if (shutdown(in, 0) < 0)
		return (-1);
	if (shutdown(out, 1) < 0)
		return (-1);
	return (x);
}
