/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* copy2way.c - copy bidirectionally on a pair of sockets
 * vixie 16may94 [moved copy1way.c into its own source file]
 * vixie 27apr94 [split closefdset into its own source file]
 * vixie 14mar94 [original]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: copy2way.c,v 1.5 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <sysexits.h>

static struct {
	pid_t	pid;
	int	status, flag;
	sig_t	old_sigchld;
} ctx;

static void
sigchld(int x) {
	pid_t pid;
	int status;

	/* try to get our own child's status. */
	pid = waitpid(ctx.pid, &ctx.status, WNOHANG);
	if (pid == ctx.pid &&
	    (WIFEXITED(ctx.status) || WIFSIGNALED(ctx.status)))
		ctx.flag++;

	/* call previous handler if there is one. */
	if (ctx.old_sigchld != SIG_IGN && ctx.old_sigchld != SIG_DFL)
		(*ctx.old_sigchld)(x);

	/* now clean up all children or we'll loop signalling. */
	while (-1 != (pid = waitpid(-1, &status, WNOHANG))) {
		/* if ours wasn't the one that caused the SIGCHLD that got us
		 * here, it could come up now (does POSIX specify whether a
		 * process which exits during a parent's previously called
		 * SIGCHLD handler is available to a waitpid()?).
		 */
		if (pid == ctx.pid &&
		    (WIFEXITED(status) || WIFSIGNALED(status)))
			ctx.flag++;
	}
}

int
copy2way(int s1, int s2, char *buf, int bufsiz) {
	int ret, save;
	sigset_t prev, just1;

	/* Must do these first since the child can run (and die) first. */
	ctx.flag = 0;
	ctx.old_sigchld = signal(SIGCHLD, sigchld);

	switch (ctx.pid = fork()) {
	case -1:
		/* error */
		(void) signal(SIGCHLD, ctx.old_sigchld);
		ret = -1;
		goto done;
	case 0:
		/* child */
		(void) signal(SIGCHLD, ctx.old_sigchld);
		if (copy1way(s1, s2, buf, bufsiz) == -1)
			exit(errno);
		exit(EX_OK);
		/*NOTREACHED*/
	default:
		/* parent */
		ret = copy1way(s2, s1, buf, bufsiz);
		if (ret == -1)
			save = errno;
		sigemptyset(&just1);
		sigaddset(&just1, SIGCHLD);
		sigprocmask(SIG_BLOCK, &just1, &prev);
		while (!ctx.flag)
			sigsuspend(&prev);
		(void) signal(SIGCHLD, ctx.old_sigchld);
		sigprocmask(SIG_SETMASK, &prev, NULL);
		if (ret == -1) {
			errno = save;
			goto done;
		}
		if (WIFEXITED(ctx.status) && WEXITSTATUS(ctx.status) > 0) {
			errno = WEXITSTATUS(ctx.status);
			ret = -1;
			goto done;
		}
		break;
	}
	ret = 0;
 done:
	return (ret);
}
