/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* copy2way_exec.c - exec a separate program to call copy2way()
 * vixie 16may94 [original]
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: copy2way_exec.c,v 1.3 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include "pathnames.h"
#include <sys/wait.h>
#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <sysexits.h>

static const char Path_Copy2way[] = _PATH_COPY2WAY;

#define MAXDIGITS 30	/* log10(MAXINT) + ~fuzzfactor */

int
copy2way_exec(int s1, int s2, int bufsiz) {
	char s1_buf[MAXDIGITS], s2_buf[MAXDIGITS], bufsiz_buf[MAXDIGITS];
	int status, ret;
	fd_set keep;
	sig_t sigchld;
	pid_t pid;

	FD_ZERO(&keep);
	FD_SET(s1, &keep);
	FD_SET(s2, &keep);
	sprintf(s1_buf, "%d", s1);
	sprintf(s2_buf, "%d", s2);
	sprintf(bufsiz_buf, "%d", bufsiz);

	sigchld = signal(SIGCHLD, SIG_DFL);

	switch (pid = vfork()) {
	case -1:
		/* error */
		ret = -1;
		goto done;
	case 0:
		/* child */
		close_fdset(&keep, 0);
		execl(Path_Copy2way, Path_Copy2way,
		      s1_buf, s2_buf, bufsiz_buf,
		      NULL);
		exit(errno);
		/*NOTREACHED*/
	default:
		/* parent */
		if (waitpid(pid, &status, 0) == -1) {
			ret = -1;
			goto done;
		}
		if (WIFEXITED(status) && WEXITSTATUS(status) > 0) {
			errno = WEXITSTATUS(status);
			ret = -1;
			goto done;
		}
		break;
	}
	ret = 0;
 done:
	(void) signal(SIGCHLD, sigchld);
	return (ret);
}
