/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sock_alen.c - what is the actual useful length of a sockaddr?
 * vixie 14may94 [cloned from sock_same.c (27apr94)]
 */

#if !defined(lint) && !defined(codecenter)
static char rcsid[] = "$Id: sock_alen.c,v 1.2 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"

/* The system has a bad habit of putting unpredictable, arbitrary junk into
 * sockaddr's it writes into via accept() and getpeername().  We need a way
 * to ensure that we only care about bytes near the front which are actually
 * filled in with good data.
 *
 * Since our purpose is to expand the meaning of the sa_len field, that's
 * what we will return if we don't know better.  This field represents the
 * total size of the sockaddr, not just the number of bytes used in sa_data.
 *
 * Note that since this function applies generally to all sockaddrs, that's
 * its argument type (rather than ``struct sock'' as is common in the rest
 * of this library.)
 */

int
sock_alen(const struct sockaddr *sa) {
#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
	struct sockaddr_in *sin;
#endif
	int guessLen;

	switch (sa->sa_family) {
#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
	case AF_INET:
		guessLen = sizeof sin->sin_len +
			   sizeof sin->sin_family +
			   sizeof sin->sin_port +
			   sizeof sin->sin_addr;
		break;
#endif
	default:
		guessLen = sizeof *sa;
		break;
	}
	if (guessLen > sa->sa_len)
		guessLen = sa->sa_len;
	return (guessLen);
}
