/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sock_aton.c - convert ascii to internal form
 * vixie 27apr94 [moved a lot of things into their own source files]
 * vixie 25feb94 [original]
 */

#if !defined(lint) && !defined(codecenter)
static char rcsid[] = "$Id: sock_aton.c,v 1.2 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

int
sock_aton(const char *src, struct sock *dst) {
	char *sep = strchr(src, '/');

#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
	if (sep && !strncmp(_sock_IP, src, sep-src)) {
		char tmp[20];
		int type, len, port;
		struct in_addr a;
		struct sockaddr_in *in;

		src = sep+1;
		if (!(sep = strchr(src, '/')))
			return (0);
		type = -1;
# if defined(_SOCK_IPTCP)
		if (!strncmp(_sock_IP_TCP, src, sep-src))
			type = SOCK_STREAM;
# endif
# if defined(_SOCK_IPUDP)
		if (!strncmp(_sock_IP_UDP, src, sep-src))
			type = SOCK_DGRAM;
# endif
		if (type == -1)
			return (0);
		src = sep+1;
		if (!(sep = strchr(src, '/')))
			return (0);

		len = sep - src;
		if (len >= sizeof(tmp))
			return (0);

		strncpy(tmp, src, len);
		tmp[len] = '\0';
		if (!strcmp(tmp, _sock_IP_Wildcard))
			a.s_addr = INADDR_ANY;
		else if (inet_aton(tmp, &a) == 0)
			return (0);

		/* we permit "0" since we use it as a wildcard */
		port = atoi(sep+1);
		if (port < 0 || port > 65535)
			return (0);

		memset(dst, 0, sizeof *dst);
		dst->type = type;
		in = (struct sockaddr_in *) &dst->addr;
		in->sin_port = htons(port);
		in->sin_addr = a;
		in->sin_family = AF_INET;
		in->sin_len = sizeof(*in);
		return (1);
	}
#endif /*IPTCP || IPUDP*/
	return (0);
}
