/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sock_bind.c - high-level wrapper around bind(2)
 * vixie 27apr94 [moved a lot of things into their own source files]
 * vixie 25feb94 [original]
 */

#if !defined(lint) && !defined(codecenter)
static char rcsid[] = "$Id: sock_bind.c,v 1.2 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

int
sock_bind(int s, struct sock *a, int f) {
#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
	struct sockaddr_in *in = (struct sockaddr_in *)(&a->addr);
#endif
	int st;

#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
	if (sock_af(a) == AF_INET &&
	    (f & SOCK_F_RSVD) &&
	    !ntohs(in->sin_port)) {
		int port = IPPORT_RESERVED;

		while (--port > 0) {
			in->sin_port = htons(port);
			st = bind(s, &a->addr, sizeof *in);
			if (st != -1 || errno != EADDRINUSE)
				break;
		}
		if (port == 0)
			errno = EAGAIN;
	}
#endif /*TCP or UDP*/
	else {
		st = bind(s, &a->addr, sizeof a->addr);
	}
	return (st);
}
