/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* sock_ntoa.c - convert internal form to ascii
 * vixie 27apr94 [moved a lot of things into their own source files]
 * vixie 25feb94 [original]
 */

#if !defined(lint) && !defined(codecenter)
static char rcsid[] = "$Id: sock_ntoa.c,v 1.2 1996/05/16 04:48:45 vixie Exp $";
#endif

#include "vixutil.h"
#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

char *
sock_ntoa(const struct sock *src) {
	static char dst[MAXHOSTNAMELEN];	/* arbitrary size */
#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
	struct sockaddr_in *in;
	const char *type;
#endif

	switch (sock_af(src)) {
#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
	case AF_INET:
		switch (src->type) {
# if defined(_SOCK_IPUDP)
		case SOCK_DGRAM:
			type = _sock_IP_UDP;
			break;
# endif
# if defined(_SOCK_IPTCP)
		case SOCK_STREAM:
			type = _sock_IP_TCP;
			break;
# endif
		default:
			type = "unknown";
			break;
		}
		in = (struct sockaddr_in *)(&src->addr);
		sprintf(dst, "ip/%s/%s/%d",
			type,
			(in->sin_addr.s_addr == INADDR_ANY)
				? _sock_IP_Wildcard : inet_ntoa(in->sin_addr),
			ntohs(in->sin_port));
		break;
#endif /*IPTCP || IPUDP*/
	default:
		strcpy(dst, "unrecognized_sockaddr_type");
		break;
	}
	return (dst);
}
