/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* vixutil.h - exports for libvixutil.a
 * vix 02mar94 [original]
 *
 * $Id: vixutil.h,v 1.6 1996/05/16 04:48:45 vixie Exp $
 */

#ifndef _VIXUTIL_H_
#define _VIXUTIL_H_

#include <stdio.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/time.h>

#define	_SOCK_IPTCP
#define	_SOCK_IPUDP

#if defined(_SOCK_IPTCP) || defined(_SOCK_IPUDP)
# include <netinet/in.h>
#endif

#ifndef __P
# ifdef __STDC__
#  define __P(x) x
# else
#  define __P(x) ()
# endif
#endif

#if !defined(TRUE) || !defined(FALSE)
# define TRUE 1
# define FALSE 0
#endif

/*-------------------- arpaParse*.c ---------------------*/

#define	ARPA1_OK	'2'
#define	ARPA1_MORE	'3'
#define	ARPA1_WARN	'4'
#define	ARPA1_ERR	'5'
#define ARPA_MAXSTR	990

#define		arpaParseCode __vix_arpaParseCode
int		arpaParseCode __P((const char *buf, int *code));
#define		arpaParseField __vix_arpaParseField
int		arpaParseField __P((const char *src, char *dst, int size));

/*----------------------- sock*.c -----------------------*/

/* recognized now:
 *	ip/tcp/DOTTEDQUAD/PORTNUM
 *	ip/udp/DOTTEDQUAD/PORTNUM
 * futures:
 *	bsd/ABSPATHNAME
 *	dna/AREA.NUMBER/OBJECTNUM
 *	sna/???
 *	apple/???
 *	xns/???
 *	ipx/???
 */

extern const char
		_sock_IP[],
		_sock_IP_TCP[],
		_sock_IP_UDP[],
		_sock_IP_Wildcard[];

struct sock {
	int		type;
	struct sockaddr	addr;
};

#define		SOCK_F_RSVD	0x0001

/* XXX - IP specific */
#define IN_WILD_ADDR(sin) ((sin).sin_addr.s_addr == INADDR_ANY)
#define	IN_WILD_PORT(sin) (ntohs((sin).sin_port) == 0)

#define		sock_af(sp) ((sp)->addr.sa_family)
#define		sock_len(sp) ((sp)->addr.sa_len)

#define		sock_ntoa __vix_sock_ntoa
char		*sock_ntoa __P((const struct sock *src));
#define		sock_ntox __vix_sock_ntox
char		*sock_ntox __P((const struct sock *src));
#define		sock_aton __vix_sock_aton
int		sock_aton __P((const char *src, struct sock *dst));
#define		sock_same __vix_sock_same
int		sock_same __P((const struct sock *s1, const struct sock *s2));
#define		sock_alen __vix_sock_alen
int		sock_alen __P((const struct sockaddr *sa));
#define		sock_isdgram __vix_sock_isdgram
int		sock_isdgram __P((const struct sock *s));
#define		sock_isstream __vix_sock_isstream
int		sock_isstream __P((const struct sock *s));
#define		sock_fdtype __vix_sock_fdtype
int		sock_fdtype __P((int fd));
#define		sock_listening __vix_sock_listening
int		sock_listening __P((int fd));
#define		sock_sock __vix_sock_sock
struct sock	sock_sock __P((int type, const struct sockaddr *sa));
#define		sock_name __vix_sock_name
int		sock_name __P((int src, struct sock *dst));
#define		sock_peer __vix_sock_peer
int		sock_peer __P((int src, struct sock *dst));
#define		sock_bind __vix_sock_bind
int		sock_bind __P((int s, struct sock *a, int f));

/*--------------------- addrhint.c ---------------------*/

/* all three of these functions return a boolean which
 * indicates whether the entry is or was already present.
 * this boolean only respects TTL in addrhint_qry().
 */

#define		addrhint_set __vix_addrhint_set
int		addrhint_set __P((struct sock addr,
				  time_t timeout,
				  void *userdata,
				  int flags));
#define		addrhint_clr __vix_addrhint_clr
int		addrhint_clr __P((struct sock addr));
#define		addrhint_qry __vix_addrhint_qry
int		addrhint_qry __P((struct sock addr,
				  time_t *timeout,
				  void **userdata,
				  int *flags));

#define	ADDRHINT_DYN	0x0001

/*---------------------- fgettok.c ---------------------*/

#define		fgettok __vix_fgettok
int		fgettok __P((FILE *f,
			     char *buf,
			     int buflen,
			     const char *sepset,
			     const char *comset));

/*--------------------- is*.c ---------------------*/

#define		str_isnumber __vix_str_isnumber
int		str_isnumber __P((const char *));
#define		str_ishexnum __vix_str_ishexnum
int		str_ishexnum __P((const char *));

/*----------------------- extoa.c ----------------------*/

#define		extoa __vix_extoa
char		*extoa __P((int status));
#define		exok __vix_exok
int		exok __P((int status));

/*---------------------- copy*way*.c --------------------*/

#define		copy1way __vix_copy1way
int		copy1way __P((int in, int out, char *buf, int bufsiz));
#define		copy2way __vix_copy2way
int		copy2way __P((int s1, int s2, char *buf, int bufsiz));
#define		copy2way_exec __vix_copy2way_exec
int		copy2way_exec __P((int s1, int s2, int bufsiz));
#define		close_fdset __vix_close_fdset
void		close_fdset __P((fd_set *fds, int isset));

/*---------------------- sleep2.c ----------------------*/

#define	S2F_NONE	0x0000	/* placeholder */
#define	S2F_INTR	0x0001	/* if sleep should be interruptable */

#define		sleep2 __vix_sleep2
int		sleep2 __P((double interval, int flags));

/*---------------------- zalloc.c ----------------------*/

#define	ZAL_NONE	0x0000	/* placeholder */
#define	ZAL_ASSERT	0x0001	/* if we should assert on allocation fails */
#define	ZAL_ZERO	0x0002	/* if the memory should be cleared to zero */

#define		zalloc __vix_zalloc
void		*zalloc __P((size_t size, unsigned int flags));

#define ZALLOC(type, flags) ((type*) zalloc(sizeof(type), flags))

/*---------------------- interval*.c ----------------------*/

struct interval {
	time_t		last;
	time_t		retry;
};

#define		interval_overdue_p __vix_interval_overdue_p
int		interval_overdue_p(register const struct interval *iv);
#define		interval_reset __vix_interval_reset
void		interval_reset(register struct interval *iv, int retry);

/*-------- list handling --------*/

#define	ListHeadV(f,t) struct { t *head, *tail; } f
#define ListHead(f,t) ListHeadV(f,t);
#define ListLinkV(t) struct { t *next; } link
#define ListLink(t) ListLinkV(t);
#define	ListInit(l) do { (l)->head = NULL; (l)->tail = NULL; } while (0)
#define	ListAdd(l,n) do { \
			n->link.next = NULL; \
			if ((l)->tail) \
			  (l)->tail->link.next = n; \
			else \
			  (l)->head = n; \
			(l)->tail = n; \
		      } while (0)
#define	ListLast_p(i) ((i)->link.next == NULL)
#define	ListEmpty_p(l) ((l)->head == NULL)
#define ListEach(l,i) for (i = (l)->head;  i;  i = i->link.next)
#define	ListEach2(l,i,t) for (i = (l)->head; \
			     (i && (t = i->link.next)), i; \
			     i = t)
#define	ListUnlink(l,i,p) do { \
			    if (p) { \
			      if (!(p->link.next = i->link.next)) \
				(l)->tail = p; \
			    } else { \
			      if (!((l)->head = i->link.next)) \
				(l)->tail = NULL; \
			    } \
			  } while (0)

/*------------------------------------------------------*/

#endif /*_VIXUTIL_H*/
