/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* main.c - mainline for proxynetd
 * vix 24feb94 [original]
 */

#if !defined(lint) && !defined(codecenter)
static char rcsid[] = "$Id: main.c,v 1.2 1996/05/16 04:48:56 vixie Exp $";
#endif

#include "externs.h"
#define _EXT
#include "defs.h"
#undef _EXT

int		DebugLevel = 0;
const char	*ProgramName = "anonymous";
char		HostName[MAXHOSTNAMELEN];

static void	usage(char *msg),
		args(int argc, char *argv[]),
		sigchld(int param);

main(int argc, char *argv[]) {
	sigset_t sigs;

	gethostname(HostName, sizeof(HostName));
	if (ProgramName = strrchr(argv[0], '/'))
		ProgramName++;
	else
		ProgramName = argv[0];
	args(argc, argv);
	(void) setsid();
	openlog(ProgramName,
		LOG_CONS | LOG_NDELAY | LOG_PID | (DebugLevel ?LOG_PERROR :0),
		LOG_LOCAL0);
	dbInit();
	signal(SIGCHLD, sigchld);
	protocol();
	sigemptyset(&sigs);
	sigaddset(&sigs, SIGKILL);
	sigprocmask(SIG_BLOCK, &sigs, NULL);
	kill(0, SIGKILL);
	sigprocmask(SIG_UNBLOCK, &sigs, NULL);
	sleep(1);
	myExit(EX_OK);
}

void
myExit(int code) {
	closelog();
	(void) exit(code);
}

static void
usage(char *msg) {
	fprintf(stderr, "%s: %s\n", ProgramName, msg);
	fprintf(stderr, "usage:  %s [-x lev]\n", ProgramName);
	myExit(EX_USAGE);
}

static void
args(int argc, char *argv[]) {
	int ch;

	while (EOF != (ch = getopt(argc, argv, "x:f:"))) {
		switch (ch) {
		case 'x':
			DebugLevel = atoi(optarg);
			break;
		default:
			usage("bad argument");
			/*NOTREACHED*/
		}
	}
}

static void
sigchld(int param) {
	pid_t pid;
	int status;

	pid = waitpid(-1, &status, WNOHANG);
	if (pid == -1)
		syslog(LOG_WARNING, "waitpid: %m");
	else if (!exok(status))
		syslog(LOG_INFO, "sigchld(%ld): %s",
		       (u_long)pid, extoa(status));
}
