/*
 * Copyright (c) 1994-1996 by Vixie Enterprises.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND VIXIE ENTERPRISES DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VIXIE ENTERPRISES BE LIABLE
 * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* session.c - session control for proxytcpd
 * vix 05may94 [moved verbs to separate file]
 * vix 24feb94 [original]
 */

#if !defined(lint) && !defined(codecenter)
static char rcsid[] = "$Id: session.c,v 1.3 1996/05/16 04:48:56 vixie Exp $";
#endif

#include "externs.h"
#include "defs.h"

static int	command(int argc, const char * const *argv,
			const struct verb verbs[]);
static char	msgBuf[ARPA_MAXSTR + 50];

void
session(void (signon)(void),
	void (signoff)(void),
	int *quitFlag,
	const struct verb verbs[]
) {
	char buf[ARPA_MAXSTR];

#ifdef debug_sessions
	{
		struct sockaddr_in sa;
		int len = sizeof sa;

		if (getsockname(STDIN_FILENO, (struct sockaddr *)&sa, &len)>=0)
			syslog(LOG_INFO, "sock [%s].%d",
			       inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
		if (getpeername(STDIN_FILENO, (struct sockaddr *)&sa, &len)>=0)
			syslog(LOG_INFO, "peer [%s].%d",
			       inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
	}
#endif
	setvbuf(stdout, NULL, _IOLBF, 0);
	(*signon)();
	*quitFlag = FALSE;
	while (fgets(buf, sizeof buf, stdin)) {
		char *bufp, **argv;
		int code, argc;

		if (bufp = strchr(buf, '\r'))
			*bufp = '\0';
		if (bufp = strchr(buf, '\n'))
			*bufp = '\0';
		argc = 0;
		argv = NULL;
		bufp = buf;
		while (bufp) {
			char *val;

			while ((val = strsep(&bufp, " \t")) && !*val)
				;
			if (val) {
				argv = realloc(argv, sizeof(char*) * (argc+1));
				if (!argv) {
					eemit("malloc", errno);
					continue;
				}
				argv[argc++] = val;
			}
		}
		code = command(argc, (const char * const *)argv, verbs);
		if (argv)
			free(argv);
		if (*quitFlag)
			break;
	}
	(*signoff)();
}

int
emit1(int code, char sep, const char *msg, int len) {
	printf("%03o%c", code, sep);
	if (len == 0)
		fputs(msg, stdout);
	else
		fwrite(msg, sizeof(char), len, stdout);
	fputs("\r\n", stdout);
	return (code);
}

int
emit(int code, const char *msg) {
	char sep = ' ';
	const char *nl;

	while (nl = strchr(msg, '\n')) {
		(void) emit1(code, sep, msg, nl - msg);
		sep = '-';
		msg = nl+1;
	}
	return (code);
}

int
eemit(const char *msg, int err) {
	sprintf(msgBuf, "%s: %s\n", msg, strerror(err));
	return (emit(0505, msgBuf));
}

int
eemita(const char *msg, int err, const struct sock *sock) {
	sprintf(msgBuf, "%s(%s|%s): %s\n",
		msg, sock_ntoa(sock), sock_ntox(sock), strerror(err));
	return (emit(0505, msgBuf));
}

void
eclose(int fd) {
	int save = errno;

	(void) close(fd);
	errno = save;
}

static int
command(int argc, const char * const *argv, const struct verb verbs[]) {
	const char *verb;
	const struct verb *v;

	verb = *argv++; argc--;
	for (v = verbs;  v->verb;  v++) {
		if (!strcasecmp(v->verb, verb)) {
			if (argc < v->minarg)
				return (emit(0501, "too few arguments\n"));
			if (argc > v->maxarg)
				return (emit(0502, "too many arguments\n"));
			return (*v->func)(argc, argv);
		}
	}
	sprintf(msgBuf, "command unrecognized <%s>\n", verb);
	return (emit(0500, msgBuf));
}
